#include "search_star.h"
#include <stdio.h>
#include <math.h>


bool RDD2XYZ (const struct RawPosition *pos,
	double &x, double &y, double &z, bool bUseGalacticCoords)
{
	double	theta, phi;
	int		dec_sign;
	double	cx, cy, cz;

	if (!pos->bDistance)
		return	false;

	if (pos->Distance == 0.0)
	{
		x = y = z = 0.0;	// special case for our sun
		return	true;
	}

	if (!pos->bRA  ||  !pos->bDec)
		return	false;

	theta = (pos->RA_hr + pos->RA_min/60.0 + pos->RA_sec/3600.0)
		* PI / 12.0;

	if (pos->Dec_deg < 0)
		dec_sign = -1;
	else
		dec_sign = 1;
	phi = (90.0 - (pos->Dec_deg + pos->Dec_arcmin*dec_sign/60.0 +
			pos->Dec_arcsec*dec_sign/3600.0)) * PI / 180.0;

	cx = pos->Distance * cos(theta) * sin(phi);
	cy = pos->Distance * sin(theta) * sin(phi);
	cz = pos->Distance * cos(phi);

	if (bUseGalacticCoords)
	{
		x = -0.0672*cx - 0.8727*cy - 0.4835*cz;
		y =  0.4927*cx - 0.4504*cy + 0.7445*cz;
		z = -0.8676*cx - 0.1884*cy + 0.4602*cz;
	}
	else
	{
		x = cx;
		y = cy;
		z = cz;
	}
	return	true;
}


bool FetchRawPosition (HSTMT hstmt, long id, struct RawPosition *pos)
{
	char	buffer[512];
	RETCODE	retcode;
	SDWORD	cbRA_hr, cbRA_min, cbRA_sec;
	SDWORD	cbDec_deg, cbDec_arcmin, cbDec_arcsec;
	SDWORD	cbDistance, cbDistanceError, cbDistanceSource;
	SDWORD	cbLastUpdated, cbEpoch;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT RA_hr, RA_min, RA_sec, Dec_deg, Dec_arcmin, "
		"Dec_arcsec, Distance, DistanceError, DistanceSource, LastUpdated, "
		"Epoch FROM Positions WHERE OwnerID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode != SQL_SUCCESS)
		return	false;

	SQLBindCol(hstmt, 1, SQL_C_UTINYINT, &pos->RA_hr, 0, &cbRA_hr);
	SQLBindCol(hstmt, 2, SQL_C_UTINYINT, &pos->RA_min, 0, &cbRA_min);
	SQLBindCol(hstmt, 3, SQL_C_FLOAT, &pos->RA_sec, 0, &cbRA_sec);
	SQLBindCol(hstmt, 4, SQL_C_SHORT, &pos->Dec_deg, 0, &cbDec_deg);
	SQLBindCol(hstmt, 5, SQL_C_UTINYINT, &pos->Dec_arcmin, 0, &cbDec_arcmin);
	SQLBindCol(hstmt, 6, SQL_C_FLOAT, &pos->Dec_arcsec, 0, &cbDec_arcsec);
	SQLBindCol(hstmt, 7, SQL_C_DOUBLE, &pos->Distance, 0, &cbDistance);
	SQLBindCol(hstmt, 8, SQL_C_DOUBLE, &pos->DistanceError, 0, &cbDistanceError);
	SQLBindCol(hstmt, 9, SQL_C_CHAR, &pos->DistanceSource, 11, &cbDistanceSource);
	SQLBindCol(hstmt, 10, SQL_C_DATE, &pos->LastUpdated, 0, &cbLastUpdated);
	SQLBindCol(hstmt, 11, SQL_C_DOUBLE, &pos->Epoch, 0, &cbEpoch);
	retcode = SQLFetch(hstmt);
	if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
		return	false;
	pos->bRA = cbRA_hr != SQL_NULL_DATA;
	pos->bDec = cbDec_deg != SQL_NULL_DATA;
	pos->bDistance = cbDistance != SQL_NULL_DATA;
	pos->bDistanceError = cbDistanceError != SQL_NULL_DATA;
	pos->bDistanceSource = cbDistanceSource != SQL_NULL_DATA;
	return	true;
}

const char *FormatWithSignificantDigits
	(double value, double standardError,
	const char *flagsWidth /* = NULL */, char *buf /* = NULL */)
{
	char formatBuf [20];
	static char sBuf[100];
	char *outBuf = buf ? buf : sBuf;

	strcpy (formatBuf, "%");
	if (flagsWidth)
		strcat (formatBuf, flagsWidth);
	if (standardError > 0.0)
	{
		int sigDigits = (int) log10 (1 / standardError) + 2;
		if (sigDigits < 1)
			sigDigits = 1;
		int precision = sigDigits - (int) log10 (fabs(value)) - 1;
		if (precision < 0)
			precision = 0;
		strcat (formatBuf, ".*f");
		sprintf (outBuf, formatBuf, precision, value);
	}
	else
	{
		strcat (formatBuf, "g");
		sprintf (outBuf, formatBuf, value);
	}
	return	outBuf;
}
